from sockjs.cyclone import basehandler
from sockjs.cyclone.transports import base


class PollingTransportBase(basehandler.PreflightHandler, base.BaseTransportMixin):
    """ Polling transport handler base class """

    def initialize(self, server):
        super(PollingTransportBase, self).initialize(server)

        self.session = None

    def _get_session(self, session_id):
        return self.server.get_session(session_id)

    def _attach_session(self, session_id, start_heartbeat=True):
        session = self._get_session(session_id)

        if session is None:
            session = self.server.create_session(session_id)

        # Try to attach to the session
        if not session.set_handler(self, start_heartbeat):
            return False

        self.session = session

        # Verify if session is properly opened
        session.verify_state()

        return True

    def _detach(self):
        """ Detach from the session """
        if self.session:
            self.session.remove_handler(self)
            self.session = None

    def check_xsrf_cookie(self):
        pass

    def send_message(self, message, stats=True):
        """ Called by the session when some data is available """
        raise NotImplementedError()

    def session_closed(self):
        """ Called by the session when it was closed """
        self._detach()

    def on_connection_close(self, reason):
        # If connection was dropped by the client, close session.
        # In all other cases, connection will be closed by the server.
        if self.session is not None:
            self.session.close(1002, 'Connection interrupted')

        super(PollingTransportBase, self).on_connection_close(reason)



import simplejson

json_encode = lambda data: simplejson.dumps(data, separators=(',', ':'))
json_decode = lambda data: simplejson.loads(data)
JSONDecodeError = ValueError

# Protocol handlers
CONNECT = 'o'
DISCONNECT = 'c'
MESSAGE = 'm'
HEARTBEAT = 'h'


# Various protocol helpers
def disconnect(code, reason):
    """Return SockJS packet with code and close reason

    @param code: Closing code
    @param reason: Closing reason
    """
    return 'c[%d,"%s"]' % (code, reason)



class BaseTransportMixin(object):
    """Base transport.

    Implements few methods that session expects to see in each transport.
    """

    name = 'override_me_please'

    def get_conn_info(self):
        from sockjs.cyclone.conn import ConnectionInfo

        """ Return C{ConnectionInfo} object from current transport """
        return ConnectionInfo(self.request.remote_ip,
                              self.request.cookies,
                              self.request.arguments,
                              self.request.headers,
                              self.request.path)

    def session_closed(self):
        """ Called by the session, when it gets closed """
        pass


class MultiplexTransport(BaseTransportMixin):
    name = 'multiplex'

    def __init__(self, conn_info):
        self.conn_info = conn_info

    def get_conn_info(self):
        return self.conn_info



#!/usr/bin/env python

from distutils.core import setup


def readfile(fname):
    with open(fname) as f:
        content = f.read()
    return content


setup(name='sockjs-cyclone',
      version='1.0.0',
      author='Flavio Grossi',
      author_email='flaviogrossi@gmail.com',
      description='SockJS python server for the Cyclone Web Server',
      license=readfile('LICENSE'),
      long_description=readfile('README.rst'),
      keywords=[ 'sockjs',
                 'cyclone',
                 'web server',
                 'websocket'
               ],
      url='http://github.com/flaviogrossi/sockjs-cyclone/',
      packages=[ 'sockjs',
                 'sockjs.cyclone',
                 'sockjs.cyclone.transports'
               ],
      requires=[ 'twisted (>=12.0)',
                 'cyclone (>=1.0)',
                 'simplejson'
               ],
      install_requires=[ 'twisted>=12.0',
                         'cyclone>=1.0-rc8',
                         'simplejson'
                       ],
      classifiers=(
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Framework :: Twisted',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
          'Topic :: Software Development :: Libraries :: Python Modules',
      )
)
